#!/usr/local/perl5/bin/perl

# This CGI script is a hack for handling "ErrorDocument 401"
# redirects when attempting Kerberos authentication via
# HTTP/SPNEGO.  It is needed because having the error handled
# by, say, a jsp page, i.e. behind a mod_jk connector, breaks
# the Negotiate option.

# Use oldstyle_urls because some browsers mishandle refresh of
# a URL with ';'.
use CGI qw/-oldstyle_urls/;

# The location to go to (the login/error jsp).  The incoming query
# string will be added to this URL to construct the refresh URL.
my $login_URL = '/shibboleth-idp/AuthSSO';

# The incoming query parameter to check that we got here via a failed
# authentication attempt.
my $login_param = 'TSClogin_kerberos';

# The name and value of the error parameter to set for the login jsp.
my $error_param_name = 'TSCerror';
my $error_param_value = 'kerberos';

my $q = CGI->new;

my $refresh_URL = '';

# Make sure we got here via an ErrorDocument redirect of an
# attempted Kerberos authentication.
if (defined $ENV{REDIRECT_QUERY_STRING} && $q->param($login_param)) {
    # Construct the URL we will refresh to.
    # Set the error parameter for the login jsp to use.
    $q->param($error_param_name, $error_param_value);
    my $query_string = $q->query_string;
    $refresh_URL = $login_URL . '?' . $query_string;
}

# Print the headers.
print $q->header (-type => 'text/html', -charset => 'utf-8',
		  -Pragma => 'no-cache',
		  -Cache_Control => 'no-cache, no-store');
 
if ($refresh_URL) {
    # Set for refresh.
    print $q->start_html(-lang => 'en',
			 -title => 'Touchstone@MIT : Kerberos Authentication Failed',
			 -head => [
				   $q->meta({-http_equiv => 'content-type',
					     -content    => 'text/html; charset=utf-8'}),
				   $q->meta({-http_equiv => 'pragma',
					     -content    => 'no-cache'}),
				   $q->meta({-http_equiv => 'refresh',
					     -content    => "0;url=$refresh_URL"}),
				   ]);

    # Add some helpful text and a link in case the user gets stuck on
    # this page.
    print <<END;
    <div class="error">
      Kerberos authentication failed.  You should be directed back to the
      Touchstone login page shortly; click
      <a href="$refresh_URL" title="Go to the Touchstone login page">here</a>
      to go there now.
    </div>
END

} else {
    # Not here via an appropriate redirect.
    print $q->start_html(-lang => 'en',
			 -title => 'Error',
			 -head => [
				   $q->meta({-http_equiv => 'content-type',
					     -content    => 'text/html; charset=utf-8'}),
				   ]);

    print 'Error - this page should not be invoked directly.';
}

print $q->end_html;
